window

package
v0.0.0-...-f8103e0 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Aug 29, 2017 License: MIT Imports: 2 Imported by: 9

Documentation

Overview

Package window provides window functions for digital signal processing.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func Hanning

func Hanning(n int) []float64

Hanning returns a periodic Hannning window of size n.

func Product

func Product(window1 []float64, window2 []float64) ([]float64, error)

Product returns the product of two windows.

If one of the windows is equal to nil, the other will be returned. If both are equal to nil, nil will be returned.

Types

This section is empty.

Jump to