Documentation
¶
Overview ¶
Package wsola implements the WSOLA (Waveform Similariy-based Overlap-Add) time-scale modification procedure.
WSOLA works in the same way as OLA, with the exception that it allows slight shift of the position of the analysis frames.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func New ¶
func New(channels int, analysisHop int, synthesisHop int, frameLength int, tolerance int) (*tsm.TSM, error)
New returns a TSM implementing the WSOLA procedure.
channels is the number of channels of the signal that the TSM will process. tolerance is the maximum number of samples that the analysis frame can be shifted. Read the documentation of the tsm.Settings type for an explanation of the other arguments.
func NewWithSpeed ¶
func NewWithSpeed(channels int, speed float64, synthesisHop int, frameLength int, tolerance int) (*tsm.TSM, error)
NewWithSpeed returns a TSM implementing the WSOLA procedure, modifying the speed of the input signal by the ratio speed.
The arguments speed, synthesisHop, frameLength and tolerance may be strictly negative, in which case they will be replaced by default values.
Types ¶
This section is empty.